import sys
import argparse
from pipetex2tex import convert

def main():
    parser = argparse.ArgumentParser(description='Convert \input{|command} to the output of command.')
    parser.add_argument('filename', type=str, nargs='?',
                        help='Files to convert',default=None)
    parser.add_argument('-e','--enc',
                        default='utf8',
                        help='encoding')

    args = parser.parse_args()

    # if no positional argument was given, read in the piped input
    if args.filename is None:
        s = sys.stdin.read()
    else:
        with open(args.filename,'r',encoding=args.enc) as f:
            s = f.read()

    converted = convert(s)

    sys.stdout.write(converted)

if __name__=="__main__":
    main()




# -*- coding: utf-8 -*-
"""
Initializes this package with metadata.
"""

from .metadata import (
        __version__,
        __author__,
        __copyright__,
        __credits__,
        __license__,
        __maintainer__,
        __email__,
        __status__,
    )

from .tools import convert



import re
from subprocess import Popen, PIPE
import sys

def _extract_command_from_within_brackets(text):

    prefixes = [' ','|','{',r'\input',r'\inp']
    while True:
        done = True
        for c in prefixes:
            if text.startswith(c):
                text = text[len(c):]
                done = False
        if done:
            break

    suffixes = [' ','}']
    while True:
        done = True
        for c in suffixes:
            if text.endswith(c):
                text = text[:-len(c)]
                done = False
        if done:
            break

    return text

def _run_command(cmd):
    proc = Popen(cmd,shell=True,stdout=PIPE,stderr=PIPE)
    out, err = proc.communicate()
    if proc.wait() != 0:
        output = "Command \"" + cmd+"\" raised errors:\n"+err.decode('utf-8')
        success = False
    else:
        output = out.decode('utf-8')
        success = True

    if output.endswith('\n'):
        output = output[:-1]
    return output, success

def convert(tex):
    r"""
    Finds all instances of a tex-command input in a
    string, runs the contained commands as subprocesses
    and replaces the tex-commands with the respective
    results of the run.

    Explicitly, it matches the input against
    the regexp (\\inp|\\input)\s*{\s*\|.*?}.

    For an explanation of the regexp, check
    https://regexr.com/72k51

    Prints out all errors by commands and raises
    a ValueError at the end if there was any errors
    in the commands.

    Parameters
    ==========
    tex : str
        The string containing tex source.

    Returns
    =======
    new_tex : str
        String that matches ``tex``, but every
        occurrence of an ``\input{|command}`` was
        replaced by the output of ``command``.
    """
    input_regexp = r"(\\inp|\\input)\s*{\s*\|.*?}"
    pattern = re.compile(input_regexp)
    pos = 0
    overall_success = True
    newtex = str(tex)

    found_matches = set()
    while True:
        m = pattern.search(tex, pos=pos)
        if m is None:
            break
        if m not in found_matches:
            span = m.span()
            this_match = str(m.group())
            command = _extract_command_from_within_brackets(this_match)
            output, success = _run_command(command)
            overall_success = overall_success and success
            if not success:
                err = ""
                err += "====================================\n"
                err += "An error occured for input command "+this_match+" at position "+str(span[0])+".\n"
                err += "Context:\n"
                err += "   ..."+tex[max(0,span[0]-20):span[1]+20]+"...\n"
                err += "this is the error message:\n"
                err += output + '\n'
                err += "====================================\n\n"
                sys.stderr.write(err)
            else:
                newtex = newtex.replace(this_match, output)

            found_matches.add(this_match)

            pos = span[1]

    if not overall_success:
        raise ValueError("There have been errors in the commands, see above.")

    return newtex



if __name__ == "__main__":

    tex = r"""
        This is a {\bf test}.

        I'll try converting this \input{|python -c "print('hi(input)')"}

        I'll try converting this \inp{|python -c "print('hi(inp)')"}
        I'll try converting this \inp  {  |    python -c "print('hi(inp)')"}
    """
    result = convert(tex)
    print(result)

    tex3 = r"\input{|python ../cookbook/example.py}dingdongwallawallabingbanf\input{|python ../cookbook/example.py}"
    result = convert(tex3)
    print(result)

    tex2 = r"""
        This is a {\bf test}.

        I'll try converting this \input{|python -c "print('hi(input)')"}

        I'll try converting this \inp{|python -c "print('hi(inp)')"}
        I'll try converting this \inp{|python "print('hi(inp)')"}
        I'll try converting this \inp  { |  python   "print('hi(inp)')"   }
    """



    convert(tex2)




from setuptools import setup, Extension
import setuptools
import os
import sys

# get __version__, __author__, and __email__
exec(open("./pipetex2tex/metadata.py").read())

setup(
    name='pipetex2tex',
    version=__version__,
    author=__author__,
    author_email=__email__,
    url='https://github.com/benmaier/pipetex2tex',
    license=__license__,
    description=r"CL tool to convert tex-files that contain \input{|command} with the actual output of these commands.",
    long_description='',
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=[
    ],
    tests_require=['pytest', 'pytest-cov'],
    setup_requires=['pytest-runner'],
    classifiers=['License :: OSI Approved :: MIT License',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7',
                 'Programming Language :: Python :: 3.8',
                 'Programming Language :: Python :: 3.9',
                 'Programming Language :: Python :: 3.10',
                 'Programming Language :: Python :: 3.11',
                 ],
    project_urls={
        'Documentation': 'http://pipetex2tex.benmaier.org',
        'Contributing Statement': 'https://github.com/benmaier/pipetex2tex/blob/master/CONTRIBUTING.md',
        'Bug Reports': 'https://github.com/benmaier/pipetex2tex/issues',
        'Source': 'https://github.com/benmaier/pipetex2tex/',
        'PyPI': 'https://pypi.org/project/pipetex2tex/',
    },
    entry_points = {
          'console_scripts': [
                  'pipetex2tex = pipetex2tex.cli:main',
              ],
        },
    include_package_data=True,
    zip_safe=False,
)
